;;
;; util-stdout.scm
;;
;; Output with possibility of complete silencing.
;;
;; ISC License
;;
;; Copyright 2023 Brmlab, z.s.
;; Dominik Pantůček <dominik.pantucek@trustica.cz>
;;
;; Permission to use, copy, modify, and/or distribute this software
;; for any purpose with or without fee is hereby granted, provided
;; that the above copyright notice and this permission notice appear
;; in all copies.
;; 
;; THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
;; WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
;; WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
;; AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
;; CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
;; OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
;; NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
;; CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
;;

(declare (unit util-stdout))

(module
 util-stdout
 (
  *stdout-quiet*
  stdout-print
  stdout-printf
  stdout-newline
  )

 (import scheme
	 (chicken base)
	 (chicken format))

 ;; If true, all stdout output is suppresed
 (define *stdout-quiet* (make-parameter #f))

 ;; Prints to stdout if not quiet
 (define (stdout-print . args)
   (when (not (*stdout-quiet*))
     (apply print args)))

 ;; Prints formatted string to stdout if not quiet
 (define (stdout-printf fmt . args)
   (when (not (*stdout-quiet*))
     (print (apply format fmt args))))

 ;; Prints newline if not quiet
 (define (stdout-newline)
   (when (not (*stdout-quiet*))
     (newline)))

 )
